#include <stdio.h>
#include <usb.h>

struct usb_bus *bus;
struct usb_device *dev;
usb_dev_handle *udev;

#define SATURN_VID 0x167B
#define SATURN_PID 0x0101

usb_dev_handle *open_device(unsigned int VID,unsigned int PID) {
	int buscount;
	int devcount;

	usb_dev_handle *tdev;
	int ret;

	tdev = NULL;

	buscount = 0;
	for (bus = usb_get_busses(); bus; bus = bus->next) {
		printf("Scanning bus: %d\n",buscount);
		devcount = 0;
    		for (dev = bus->devices; dev; dev = dev->next) {
			printf("Scanning device: %d P,V: 0x%04X,0x%04X\n",
				devcount,dev->descriptor.idVendor,
				dev->descriptor.idProduct);
		
			/*
			 * Try to open the device.
			 */	
      			tdev = usb_open(dev);

			if (tdev) {
				if ( (dev->descriptor.idVendor == VID) &&
				     (dev->descriptor.idProduct == PID) ) {

					printf("Device found.\n");
					return(tdev);

				}
				
			}
		}
		buscount++;
	}
	return(NULL);
}

int main(int argc,char **argv) {

unsigned char incoming[4096];
int result;
int count;
int count2;
int count3;

int bmRequestType;
int bRequest;
int wValuefrom;
int wValueto;
int wIndex;
int wLength;

	if (argc < 7) {
		printf("Usage: CVStest <bmRquestType> <bRequest>"
		       " <wValuefrom> <wValueto> <wIndex> <wLength>\n");
		printf("Example: CVStest 0xC0 0x00 0x0000 0x0100 0x0000 4096\n");
		exit(0);	
	}

	sscanf(argv[1],"0x%02X",&bmRequestType);
	sscanf(argv[2],"0x%02X",&bRequest);
	sscanf(argv[3],"0x%04X",&wValuefrom);
	sscanf(argv[4],"0x%04X",&wValueto);
	sscanf(argv[5],"0x%04X",&wIndex);
	sscanf(argv[6],"%d",&wLength);

	printf("Found bmRequestType: 0x%02X\n",bmRequestType & 0xFF);
	printf("Found bRequest arg: 0x%02X\n",bRequest & 0xFF);
	printf("Found wValue from arg: 0x%04X\n",wValuefrom & 0xFFFF);
	printf("Found wValue to arg: 0x%04X\n",wValueto & 0xFFFF);
	printf("Found wIndex arg: 0x%04X\n",wIndex);
	printf("Found wLength %d\n",wLength);

	/*
	 * Immediate printf.
	 */
	setbuf(stdout,0);  

	/*
	 * USB init.
	 */
	usb_init();
	// usb_set_debug(127);
	usb_find_busses();
	usb_find_devices();

	/*
	 * Try to open the Saturn.
	 */
	udev = open_device(SATURN_VID,SATURN_PID);

	if (!udev) {
		printf("Device not found. Exiting.\n");
		exit(0);
	}
	
	// Set configuration 1,interface 0,altinterface 0.
	usb_set_configuration(udev,1);
	usb_claim_interface(udev,0);

	// Test code goes here.
	for (count=wValuefrom;count<=wValueto;count++) {
		// Clear the incoming array.
		memset(incoming,0x00,wLength);

		result = usb_control_msg(udev,
			  bmRequestType&0xFF,
			  bRequest&0xFF,
			  count,
			  wIndex,
			  incoming,
			  wLength,
			  100);

		if (result >= 0) {
			printf("Response %d, wValue 0x%04X\n",result,count);
			for (count2=0;count2<result;count2++) {
				printf("%02X ",incoming[count2]);
				if (count2%16 == 15) {
					for (count3=(count2-15);count3 <= count2;count3++) {
						if ( (incoming[count3] > 0x1F) && (incoming[count3] < 0xF0) ) {
						printf("%c",incoming[count3]);
	} else {
		printf(".");
	}
					}
					
					printf("\n");
				}
			}
			printf("\n");
		}
	}

  	// Release.
	usb_release_interface(udev,0);

	// Finally close the device.
	if (udev) {
		usb_close (udev);
  	}
	return 0;
}

